/**
 * Copyright (c) 2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jbi.descriptor;

import java.io.IOException;
import java.net.URL;

import javax.xml.XMLConstants;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.xml.sax.SAXException;

/**
 * 
 * @author vnoel
 *
 */
public abstract class AbstractJAXBSingleton {

    /**
     * The JAXB unique unmarshaller.
     */
    private Unmarshaller unmarshaller;

    /**
     * The JAXB unique marshaller.
     */
    private Marshaller marshaller;

    private Validator validator;

    /**
     * We need an init class because it is not possible to catch exceptions from the super constructor in an extending
     * class
     */
    protected void init(Class<?>[] clazzes, String[] schemas) throws IOException, SAXException, JAXBException {
        final SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

        // The JBI schema resource is in the Jar where the class is
        // loaded
        final StreamSource[] schemasSources = new StreamSource[schemas.length];
        for (int i = 0; i < schemas.length; i++) {
            final URL schemaUrl = this.getClass().getResource("/" + schemas[i]);
            if (schemaUrl == null) {
                throw new IOException("Can't find " + schemas[i] + " on the classpath");
            }
            schemasSources[i] = new StreamSource(schemaUrl.openStream());
        }
        final Schema schema = factory.newSchema(schemasSources);

        final JAXBContext jaxbContext = JAXBContext.newInstance(clazzes);

        // Unmarshaller initialization
        unmarshaller = jaxbContext.createUnmarshaller();
        unmarshaller.setSchema(schema);

        // Marshaller initialization
        marshaller = jaxbContext.createMarshaller();
        marshaller.setSchema(schema);
        marshaller.setProperty(Marshaller.JAXB_ENCODING, "UTF-8");
        marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

        validator = schema.newValidator();
    }

    protected void init(Class<?> clazz, String... schemas) throws IOException, SAXException, JAXBException {
        this.init(new Class<?>[] { clazz }, schemas);
    }

    protected Marshaller getMarshaller() {
        return marshaller;
    }

    protected Unmarshaller getUnmarshaller() {
        return unmarshaller;
    }

    protected Validator getValidator() {
        return validator;
    }
}
